/*****************************************************************************
** DOC9.PRG
**
** ProVision:Windows v1.20 sample from Chapter 9 - STYLE GUIDE
**
** by J. David Reynolds
**
** Copyright 1991, 1992 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
*****************************************************************************/

#include "inkey.ch"
#include "pw.ch"


STATIC   oMainWnd                      // the main window



/*****************************************************************************
** DocMain( cParam1, cParam2 ) --> NIL
*****************************************************************************/
FUNCTION DocMain( cParam1, cParam2 )

   LOCAL nCols, ;
         nRows := 0

   LOCAL oColors


   // Validate the command line parameters, if any.
   IIF(cParam1 == NIL, cParam1 := "", )
   IIF(cParam2 == NIL, cParam2 := "", )
   cParam1  := UPPER(cParam1 + cParam2)

   // Switch video modes if requested and supported.
   IF "/43" $ cParam1
      nRows := MAXROW() + 1
      nCols := MAXCOL() + 1
      SETMODE(43, 80)
   END IF // "/43" $ cParam1
   IF "/50" $ cParam1
      nRows := MAXROW() + 1
      nCols := MAXCOL() + 1
      SETMODE(50, 80)
   END IF // "/50" $ cParam1

   // Init ProVision:Windows.
   IF "/STD" $ cParam1
      INIT WINDOWS ;
         STANDARD MOUSE ;
         STANDARD BORDERS ;
         STANDARD CONTROLS ;
         SETBLINK .T.
   ELSE
      INIT WINDOWS ;
         GRAPHIC MOUSE ;
         GRAPHIC BORDERS ;
         GRAPHIC CONTROLS
   END IF // "/STD" $ cParam1

   IF pw():isTMR(PWTMR_BORDERS)
      // Load new wallpaper.
      IF (pwMaxRow() == 42) .OR. (pwMaxRow() == 49)
         pw():wallpaper(PWWP_GRID, "W/N")
      ELSE
         pw():wallpaper(PWWP_RINGS, IIF(ISCOLOR(), "W/*N", "W/N"))
      END IF // (pwMaxRow() == 42) .OR. (pwMaxRow() == 49)
   ELSE
      // Clear the screen.
      SETCOLOR("+W/N")
      CLEAR SCREEN
   END IF // pw():isTMR(PWTMR_BORDERS)

   // By default, you cannot use Alt+C or or Alt+D as accelerator keys or
   // shortcut keys since Clipper traps these.  To use these keys,
   // uncomment the two lines below.  Note that by doing so, you will not
   // be able to abort the application with Alt+C nor invoke the debugger
   // with Alt+D.
// SETCANCEL(.F.)
// ALTD(0)

   // Install a system filter for keypresses.
   pw():addFilter({ | idMsg, nKey | SysKeyFilter(@idMsg, nKey) }, ;
      PWMSG_KEYPRESS)

   // Define a new color scheme.
   oColors                    := pw():getColorScheme()

   oColors:windowColors       := "W/N, N/*W, N/*W, W/W, N/W, N/W"
   oColors:menuColors         := "+W/W, +W/N, N/W, N/W, W/N"
   oColors:systemmenuColors   := "+W/W, +W/N, N/W, N/W, W/N"

   oColors:browseboxColors    := "N/W, N/W, +W/N, W/N, N/*W, W, +N/N"
   oColors:checkboxColors     := "+W/N, N/W, W/N, +N/N"
   oColors:checklistColors    := "N/W, N/W, +W/N, W/N, N/*W, W, +N/N"
   oColors:comboboxColors     := "N/W, N/W, +W/N, W/N, N/*W, W, +N/N"
   oColors:listboxColors      := "N/W, N/W, +W/N, W/N, N/*W, W, +N/N"
   oColors:pushbuttonColors   := "N/W, N/*W, +B/W, +W, +N, N/*W, +N/W"
   oColors:radiobuttonsColors := "+W/N, N/W, W/N, +N/N"
   oColors:spinnerColors      := "N/W, N/W, +W/N, W/N, +N/N"
   oColors:textfieldColors    := "N/W, N/W, +W/N, W/N, +N/N"

   // Open the main window (behind the About window).
   MainWnd()

   // Activate the ProVision:Windows event system.
   ACTIVATE WINDOWS

   // Deinit ProVision:Windows.
   DEINIT WINDOWS

   // Try to switch back to the original mode, if necessary.
   IF nRows != 0
      SETMODE(nRows, nCols)
   END IF // nRows != 0

   RETURN(NIL)
   // END DocMain( cParam1, cParam2 )



/*****************************************************************************
** STATIC MainWnd() --> oMainWnd
*****************************************************************************/
STATIC FUNCTION MainWnd()

   LOCAL oMenu


   // Turn the mouse cursor into an hourglass (or display a wait message).
   WaitMsg(.T.)

   // Create the main window.
   CREATE WINDOW oMainWnd ;
      CENTERED ;
      SIZE 12, 40 ;
      STYLE PWSTYLE_PRIMARY ;
      TITLE "ProVision:Windows menus" ;
      ICON TITLE "Menus"

   // Create the menu for the window.
   MENU oMenu
      POPUP PROMPT "~File"
         MENUITEM PROMPT "~New"
         MENUITEM PROMPT "~Open...\tCtrl+O"
         MENUITEM PROMPT "~Close"
         MENUITEM PROMPT "~Save\tCtrl+S"
         MENUITEM PROMPT "Save ~as..."
         SEPARATOR
         MENUITEM PROMPT "~Print..."
         MENUITEM PROMPT "Print set~up..."
         SEPARATOR
         MENUITEM PROMPT "E~xit\tAlt+F4"
      END POPUP
      POPUP PROMPT "~Edit"
      END POPUP
      POPUP PROMPT "~Options"
      END POPUP
   END MENU
   ATTACH MENU oMenu TO oMainWnd

   // Open the window.
   OPEN WINDOW oMainWnd

   // Restore the mouse cursor (or remove the wait message).
   WaitMsg(.F.)

   // Return a reference to the new window.
   RETURN(oMainWnd)
   // END MainWnd()



/*****************************************************************************
** STATIC SysKeyFilter( idMsg, nKey ) --> NIL
*****************************************************************************/
STATIC FUNCTION SysKeyFilter( idMsg, nKey )


   IF nKey == K_ALT_X

      IF oMainWnd:close()
         idMsg := PWMSG_NONE
      END IF // oMainWnd:close()

   ELSEIF nKey == K_ALT_F11

      _pvTxt2Grph()

   END IF // nKey == K_ALT_X

   RETURN(NIL)
   // END SysKeyFilter( idMsg, nKey )



/*****************************************************************************
** WaitMsg( lToggle ) --> NIL
*****************************************************************************/
FUNCTION WaitMsg( lToggle )

   STATIC   cScreen

   STATIC   lUseMouse

   STATIC   nOpen := 0

   LOCAL oOutput


   // Determine if ProVision:Windows initialized a mouse driver and
   // if the mouse cursor is in TMR mode.
   IF lUseMouse == NIL
      lUseMouse   := pw():isMouse() .AND. pw():isTMR(PWTMR_MOUSE)
   END IF // lUseMouse == NIL

   IF lToggle

      // Keep track of the number of calls to allow nesting.
      ++nOpen

      // Only change the state if this is the beginning of an outer-level
      // call.
      IF nOpen == 1

         // If a mouse was initialized in TMR mode, turn the cursor to
         // an hourglass, otherwise display a wait message on the
         // wallpaper.
         IF lUseMouse
            pw():mouseCursor(PMCURSOR_HOUR)
         ELSE
            oOutput  := pw():setOutput(NIL)
            cScreen  := SAVESCREEN(00, MAXCOL() - 05, 00, MAXCOL())
            SETCOLOR("*N/W")
            @ 00, MAXCOL() - 05 SAY " WAIT "
            pw():setOutput(oOutput)
         END IF // lUseMouse

      END IF // nOpen == 1

   ELSE

      // Keep track of the number of calls to allow nesting.
      --nOpen

      // Only change the state if this is the end of an outer-level call.
      IF nOpen == 0

         // If a mouse was initialized in TMR mode, restore the cursor to
         // an arrow, otherwise remove the wait message from the
         // wallpaper.
         IF lUseMouse
            pw():mouseCursor(PMCURSOR_ARROW)
         ELSE
            oOutput  := pw():setOutput(NIL)
            RESTSCREEN(00, MAXCOL() - 05, 00, MAXCOL(), cScreen)
            pw():setOutput(oOutput)
         END IF // lUseMouse

      END IF // nOpen == 0

   END IF // lToggle

   RETURN(NIL)
   // END WaitMsg( lToggle )

