/*****************************************************************************
** MCDESIGN.PRG
**
** Interactive ProVision:Windows graphical mouse cursor designer
**
** by Christopher Key and J. David Reynolds
**
** Copyright 1991-1994 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
*****************************************************************************/

#include "pw.ch"
#include "inkey.ch"

#define CRLF Chr(13)+Chr(10)

STATIC aScreenMask :={}, aCursorMask := {}

STATIC oXHot, oYHot
STATIC oOutFile, oCursorName
STATIC oMainWnd

STATIC nXOldHot := 1, nYOldHot := 1


/****************************************************************************
** FUNCTION DesignCursor()
** Parameters  - NONE
** Returns     - NIL
**
** This is the main function of mcDesigner.  It sets up the main window, and
** does all of the initialization stuff.
**
******************************************************************************/
FUNCTION DesignCursor()
STATIC oTest
LOCAL GetList := {}
LOCAL n

   //Allow Alt-C as a hot key
   SETCANCEL(.F.)

   //Initialize masks to default
   FOR n := 1 TO 16
      AADD(aScreenMask, "11111111")
      AADD(aCursorMask, "00000000")
   NEXT

   //Assume graphics mode.  would be fairly pointless in standard mode
   INIT WINDOWS

   //would also be fairly pointless in standard mode or on a system
   //without a mouse
   IF !pw():isTMR(PWTMR_MOUSE) .OR. !pw():isMouse()
      AbortMessage()
   ENDIF

   CREATE WINDOW oMainWnd CENTERED size 21, 70 ;
      STYLE PWSTYLE_PRIMARY-PWSTYLEINFO_MENU ;
      TITLE "Mouse Cursor Designer"

   oMainWnd:addFilter({|idMsg, _1, _2, _3, _4| ;
         mouseEventHandler(idMsg, _1, _2, _3, _4)}, ;
         PWMSG_MOUSEPRESS + PWMSG_MOUSERELEASE + PWMSG_MOUSEMOVEMENT)

   @ 0, 1 SAY "Screen Mask  Cursor Mask  Combined"

   //Draw the boxes
   pwFancyBox(1,1,  18,10, "W, B, +W, +W, +W, +W")
   pwFancyBox(1,14, 18,23, "N, B, +W, +W, +W, +W")
   pwFancyBox(1,27, 18,36, "B, B, +W, +W, +W, +W")

   //Do the gets
   @ 1,  39 GET AS TEXTFIELD oXHot ;
      PROMPT " ~X,Y hotspot: " ;
      PICTURE "##" ;
      INITIAL 0 ;
      VALID {||IIF(oXHot:getValue()>7 .OR. oXHot:getValue()<0, ;
               (TONE(40, 6), oXHot:setValue(nXOldHot-1)), ), UpdateHotSpot()}

   @ 1,  55 GET AS TEXTFIELD oYHot ;
      PROMPT "," ;
      PICTURE "##" ;
      INITIAL 0 ;
      VALID {||IIF(oYHot:getValue()>19 .OR. oYHot:getValue()<0, ;
               (TONE(40, 6), oYHot:setValue(nYOldHot-1)), ), UpdateHotSpot()}

   @ 2,  39 GET AS TEXTFIELD oOutFile ;
      PROMPT " ~Output file: " ;
      PICTURE "@!S12" ;
      INITIAL SPACE(64)

   @ 3,  39 GET AS TEXTFIELD oCursorName ;
      PROMPT " Cursor ~name: " ;
      PICTURE "@S12" ;
      INITIAL SPACE(32)

   @ 04, 45 GET AS PUSHBUTTON oTest ;
      PROMPT "  ~Test   " ;
      ACTION {|| MakeCursor()}

   @ 07, 45 GET AS PUSHBUTTON ;
      PROMPT "~Generate " ;
      ACTION {|| IIF(EMPTY(oCursorName:getValue()), ;
               (TONE(40, 6), oMainWnd:select(oCursorName)), ;
               oMainWnd:select(IIF(Generate(), oTest, oOutFile)))}

   @ 10, 45 GET AS PUSHBUTTON ;
      PROMPT "~Shapes..." ;
      ACTION {|| IIF(GetDefault(), (DisplayCursor(aScreenMask, 1, 2), ;
                 DisplayCursor(aCursorMask, 1, 15), DispCombined()), ), ;
                 oMainWnd:select(oTest)}

   @ 13, 45 GET AS PUSHBUTTON ;
      PROMPT "  ~Clear  " ;
      ACTION {|| SetDefault("NONE"), DisplayCursor(aScreenMask, 1, 2), ;
                 DisplayCursor(aCursorMask, 1, 15), DispCombined(), ;
                 oMainWnd:select(oTest)}

   @ 16, 45 GET AS PUSHBUTTON ;
      PROMPT "  ~Quit   " ;
      ACTION {|| oMainWnd:close()}

   DisplayCursor(aScreenMask, 1, 2)
   DisplayCursor(aCursorMask, 1, 15)
   DispCombined()

   ATTACH CONTROLS TO oMainWnd DEFAULT ACTION oTest

   OPEN WINDOW oMainWnd

   ACTIVATE WINDOWS
   DEINIT WINDOWS

RETURN NIL


/****************************************************************************
** FUNCTION MouseEventHandler()
** Parameters  - standard five event parameters
** Returns     - NIL
**
** This is the Mouse Event handler that handles drawing the cursor masks based
** on what you do.  Of primary interest is the way of using the mouse while a
** button is pressed.
**
******************************************************************************/
STATIC FUNCTION MouseEventHandler(idMsg, xParm1, xParm2, xParm3, xParm4)
STATIC lMovingHotspot := .F.
LOCAL nWindowRow, nWindowCol, nCursorRow, nCursorCol, nOldCursor

   DO CASE

      //Allow either a mouse press or movement with a button pressed to draw
      CASE (idMsg==PWMSG_MOUSEPRESS) .OR. ((idMsg==PWMSG_MOUSEMOVEMENT) ;
            .AND. (pvGetBit(xParm4, PMSTATE_LEFT) .OR. ;
            pvGetBit(xParm4, PMSTATE_RIGHT)))

         nCursorRow := ROW()
         nCursorCol := COL()
         nOldCursor := SETCURSOR(0)

         oMainWnd:ScreenToWnd(xParm2, xParm3, @nWindowRow, @nWindowCol)

         //Figure out what to do
         IF (nWindowRow>1) .AND. (nWindowRow<18)
            IF (nWindowCol>1) .AND. (nWindowCol<10)

               IF pvGetBit(xParm4, PMSTATE_LEFT)
                  SetBit(aScreenMask, nWindowRow-1, nWindowCol-1)
               ELSEIF pvGetBit(xParm4, PMSTATE_RIGHT)
                  ResetBit(aScreenMask, nWindowRow-1, nWindowCol-1)
               END

               DisplayBit(aScreenMask, 1, 2, nWindowRow-1, nWindowCol-1)
               DispBitCombined(nWindowRow-1, nWindowCol-1)

            ELSEIF (nWindowCol>14) .AND. (nWindowCol<23)

               IF pvGetBit(xParm4, PMSTATE_LEFT)
                  SetBit(aCursorMask, nWindowRow-1, nWindowCol-14)
               ELSEIF pvGetBit(xParm4, PMSTATE_RIGHT)
                    ResetBit(aCursorMask, nWindowRow-1, nWindowCol-14)
               END

               DisplayBit(aCursorMask, 1, 15, nWindowRow-1, nWindowCol-14)
               DispBitCombined(nWindowRow-1, nWindowCol-14)

            ELSEIF (nWindowCol>27) .AND. (nWindowCol<36)

               //Mouse press on hotspot allows it to be dragged (drug?).
               IF (idMsg==PWMSG_MOUSEPRESS) .AND. ;
                     ((nWindowRow-2)==oYHot:getValue()) .AND. ;
                     ((nWindowCol-28)==oXHot:getValue())
                  lMovingHotspot := .T.
               END

               DO CASE
                  CASE lMovingHotspot
                     oYHot:setValue(nWindowRow-2)
                     oXHot:setValue(nWindowCol-28)
                     UpdateHotSpot()
                  CASE pvGetBit(xParm4, PMSTATE_LEFT) .AND.  pvGetBit(xParm4, PMSTATE_RIGHT)
                     ResetBit(aCursorMask, nWindowRow-1, nWindowCol-27)
                     SetBit(aScreenMask, nWindowRow-1, nWindowCol-27)
                  CASE pvGetBit(xParm4, PMSTATE_LEFT) .AND. !pvGetBit(xParm4, PMSTATE_RIGHT)
                     SetBit(aCursorMask, nWindowRow-1, nWindowCol-27)
                     ResetBit(aScreenMask, nWindowRow-1, nWindowCol-27)
                  CASE !pvGetBit(xParm4, PMSTATE_LEFT) .AND.  pvGetBit(xParm4, PMSTATE_RIGHT)
                     ResetBit(aCursorMask, nWindowRow-1, nWindowCol-27)
                     ResetBit(aScreenMask, nWindowRow-1, nWindowCol-27)
               END

               DisplayBit(aScreenMask, 1, 2, nWindowRow-1, nWindowCol-27)
               DisplayBit(aCursorMask, 1, 15, nWindowRow-1, nWindowCol-27)
               DispBitCombined(nWindowRow-1, nWindowCol-27)

            END
         END

         DEVPOS(nCursorRow, nCursorCol)
         SETCURSOR(nOldCursor)

      //Stop dragging hotspot.
      CASE (idMsg==PWMSG_MOUSERELEASE)
         IF lMovingHotspot
            lMovingHotspot := .F.
         END

   ENDCASE

RETURN NIL

/****************************************************************************
** FUNCTION DisplayCursor()
** Parameters  -  aRows       -> array mask
**                nTop, nLeft -> upper left coordinates to draw at
** Returns     -  NIL
**
** This function is used to display an entire cursor mask.  It loops through
** the array and displays based upon which bits are set.
**
******************************************************************************/
STATIC FUNCTION DisplayCursor(aRows, nTop, nLeft)
LOCAL n, m
   FOR n := 1 TO LEN(aRows)
      FOR m := 1 to 8
         IF pvGetBit(pvBin2Num(aRows[n]), 8-m)
            @ nTop+n, nLeft+m-1 SAY " " COLOR "N/W"
         ELSE
            @ nTop+n, nLeft+m-1 SAY " " COLOR "W/N"
         END
      NEXT
   NEXT
RETURN NIL

/****************************************************************************
** FUNCTION DisplayBit()
** Parameters  -  aRows       -> cursor mask
**                nTop, nLeft -> upper left hand corner
**                nRow, nCol  -> offsets into mask
** Returns     -  NIL
**
** This function displays a single bit in a single mask
**
******************************************************************************/
STATIC FUNCTION DisplayBit(aRows, nTop, nLeft, nRow, nCol)
   IF pvGetBit(pvBin2Num(aRows[nRow]), 8-nCol)
      @ nTop+nRow, nLeft+nCol-1 SAY " " COLOR "N/W"
   ELSE
      @ nTop+nRow, nLeft+nCol-1 SAY " " COLOR "W/N"
   END
RETURN NIL

/****************************************************************************
** FUNCTION DispCombined()
** Parameters  -  NONE
** Returns     -  NIL
**
** This function is used to display the complete combined cursor mask.
** The colors are assigned as follows:
**                Cursor   Screen
**                Set      Set      Green <- probably don't want any green
**                Set      ReSet    Black <- cursor foreground
**                ReSet    Set      Blue  <- screen color
**                ReSet    ReSet    White <- cursor background
**
******************************************************************************/
STATIC FUNCTION DispCombined()
LOCAL n, m
   FOR n := 1 TO 16
      FOR m := 1 to 8
         DO CASE
            CASE  pvGetBit(pvBin2Num(aScreenMask[n]), 8-m) .AND.  pvGetBit(pvBin2Num(aCursorMask[n]), 8-m)
               @ 1+n, 27+m SAY " " COLOR "+W/G"
            CASE !pvGetBit(pvBin2Num(aScreenMask[n]), 8-m) .AND.  pvGetBit(pvBin2Num(aCursorMask[n]), 8-m)
               @ 1+n, 27+m SAY " " COLOR "+W/W"
            CASE  pvGetBit(pvBin2Num(aScreenMask[n]), 8-m) .AND. !pvGetBit(pvBin2Num(aCursorMask[n]), 8-m)
               @ 1+n, 27+m SAY " " COLOR "+W/B"
            CASE !pvGetBit(pvBin2Num(aScreenMask[n]), 8-m) .AND. !pvGetBit(pvBin2Num(aCursorMask[n]), 8-m)
               @ 1+n, 27+m SAY " " COLOR "+W/B*"
         END
      NEXT
   NEXT
   @ 1+HotY(), 27+HotX() SAY " " COLOR "+W/R"
RETURN NIL

/****************************************************************************
** FUNCTION DispBitCombined()
** Parameters  -  nRow, nCol -> offset of bit to display
** Returns     -  NIL
**
** This function displays a single bit of the combined cursor using the
** color scheme described above.
**
******************************************************************************/
STATIC FUNCTION DispBitCombined(nRow, nCol)
   DO CASE
      CASE HotY()==nRow .AND. HotX()==nCol
         @ 1+nRow, 27+nCol SAY " " COLOR "+W/R"
      CASE  pvGetBit(pvBin2Num(aScreenMask[nRow]), 8-nCol) .AND.  pvGetBit(pvBin2Num(aCursorMask[nRow]), 8-nCol)
         @ 1+nRow, 27+nCol SAY " " COLOR "+W/G"
      CASE !pvGetBit(pvBin2Num(aScreenMask[nRow]), 8-nCol) .AND.  pvGetBit(pvBin2Num(aCursorMask[nRow]), 8-nCol)
         @ 1+nRow, 27+nCol SAY " " COLOR "+W/W"
      CASE  pvGetBit(pvBin2Num(aScreenMask[nRow]), 8-nCol) .AND. !pvGetBit(pvBin2Num(aCursorMask[nRow]), 8-nCol)
         @ 1+nRow, 27+nCol SAY " " COLOR "+W/B"
      CASE !pvGetBit(pvBin2Num(aScreenMask[nRow]), 8-nCol) .AND. !pvGetBit(pvBin2Num(aCursorMask[nRow]), 8-nCol)
         @ 1+nRow, 27+nCol SAY " " COLOR "+W/B*"
   ENDCASE
RETURN NIL

/****************************************************************************
** FUNCTION SetBit()
** Parameters  -  aMask  -> cursor mask to modify
**                nIndex -> row to modify
**                nPos   -> column to modify
** Returns     -  NIL
**
** This function sets a single bit in a cursor mask
**
******************************************************************************/
STATIC FUNCTION SetBit(aMask, nIndex, nPos)
LOCAL cTemp := substr(aMask[nIndex], nPos, 1)
   aMask[nIndex] := LEFT(aMask[nIndex], nPos-1) + ;
                    "1" + ;
                    RIGHT(aMask[nIndex], 8-nPos)
RETURN NIL

/****************************************************************************
** FUNCTION ResetBit()
** Parameters  -  see SetBit() above
** Returns     -  NIL
**
** This function Resets a single bit in a cursor mask.
**
******************************************************************************/
STATIC FUNCTION ResetBit(aMask, nIndex, nPos)
LOCAL cTemp := substr(aMask[nIndex], nPos, 1)
   aMask[nIndex] := LEFT(aMask[nIndex], nPos-1) + ;
                    "0" + ;
                    RIGHT(aMask[nIndex], 8-nPos)
RETURN NIL

/****************************************************************************
** FUNCTION HotX()
** Parameters  -  NONE
** Returns     -  x Value of hot spot
**
******************************************************************************/
STATIC FUNCTION HotX()
RETURN(oXHot:getValue()+1)

/****************************************************************************
** FUNCTION HotY()
** Parameters  -  NONE
** Returns     -  y Value of hot spot
**
******************************************************************************/
STATIC FUNCTION HotY()
   RETURN(oYHot:getValue()+1)

/****************************************************************************
** FUNCTION MakeCursor()
** Parameters  -  NONE
** Returns     -  NIL
**
** This function is used to test the designed cursor.  It sets the mouse
** cursor to the currently displayed shape.  Any mouse press will reset the
** cursor to an arrow.
**
******************************************************************************/
STATIC FUNCTION MakeCursor()
STATIC nFilter
LOCAL cScreenMask:= "", cCursorMask:= "", n

   FOR n := 1 TO 16
      cScreenMask += CHR(pvBin2Num(aScreenMask[n]))
      cCursorMask += CHR(pvBin2Num(aCursorMask[n]))
   NEXT

   pw():mouseDefine( cScreenMask, cCursorMask, oXHot:getValue(), oYHot:getValue())

   nFilter := pw():addFilter({|| pw():delFilter(nFilter), ;
                              pw():mouseCursor(PMCURSOR_ARROW)}, ;
                              PWMSG_EVENT - PWMSG_MOUSEMOVEMENT)

RETURN   NIL

/****************************************************************************
** FUNCTION UpdateHotSpot()
** Parameters  -  NONE
** Returns     -  .T.
**
** This function moves the hot spot to the currently selected location.
**
******************************************************************************/
STATIC FUNCTION UpdateHotSpot()
LOCAL nCursorRow, nCursorCol, nOldCursor

   nCursorRow := ROW()
   nCursorCol := COL()
   nOldCursor := SETCURSOR(0)

   DispBitCombined(nYOldHot, nXOldHot)
   DispBitCombined(HotY(), HotX())

   nXOldHot := HotX()
   nYOldHot := HotY()

   DEVPOS(nCursorRow, nCursorCol)
   SETCURSOR(nOldCursor)

RETURN .T.  //For ease of use in Valid

/****************************************************************************
** FUNCTION Generate()
** Parameters  -  NONE
** Returns     -  Logical indicating whether or not the file was created.
**
** This function will generate a program file containing a function to
** select the cursor you designed.  The file will contain one function, that
** when called will change the mouse cursor to the one that you designed.
**
**
******************************************************************************/
STATIC FUNCTION Generate()
LOCAL nHandle, n, cFilename := ALLTRIM(oOutFile:getValue())

   IF .NOT. (".PRG" $ cFilename)
      cFilename += ".PRG"
   END

   IF FILE(cFileName)
      IF !OverWriteFile()
         RETURN .F.
      END
   END

   nHandle :=FCREATE(cFilename)

   IF nHandle==-1    // Lazy man's error checking.  Improve later.
      TONE(40, 6)
      RETURN(.F.)
   END

   FWRITE(nHandle, "/**************************************************************************"+CRLF)
   FWRITE(nHandle, " * "+PADR(ALLTRIM(cFilename), 71)+"*"+CRLF)
   FWRITE(nHandle, " * Generated by MCDesign, Written by Christopher Key                      *"+CRLF)
   FWRITE(nHandle, " *                        Modified by J. David Reynolds                   *"+CRLF)
   FWRITE(nHandle, " **************************************************************************/"+CRLF+CRLF)
   FWRITE(nHandle, "FUNCTION "+ALLTRIM(oCursorName:getValue())+"()"+CRLF)
   FWRITE(nHandle, "pw():mouseDefine( ;"+CRLF)

   FOR n := 1 TO 16
      FWRITE(nHandle, "   CHR("+STR(pvBin2Num(aScreenMask[n]))+")")
      IF n<16
         FWRITE(nHandle, "+; //"+aScreenMask[n]+CRLF)
      ELSE
         FWRITE(nHandle, ",; //"+aScreenMask[n]+CRLF)
      END
   NEXT

   FOR n := 1 TO 16
      FWRITE(nHandle, "   CHR("+STR(pvBin2Num(aCursorMask[n]))+")")
      IF n<16
         FWRITE(nHandle, "+; //"+aCursorMask[n]+CRLF)
      ELSE
         FWRITE(nHandle, ",; //"+aCursorMask[n]+CRLF)
      END
   NEXT

   FWRITE(nHandle, STR(oXHot:getValue())+", "+STR(oYHot:getValue())+")"+CRLF)
   FWRITE(nHandle, "RETURN NIL"+CRLF)

   FCLOSE(nHandle)

RETURN(.T.)

/****************************************************************************
** FUNCTION GetDefault()
** Parameters  -  NONE
** Returns     -  lSelect, whether or not a cursor was selected
**
** This function allows you to load one of ProVision:Windows' predefined
** mouse cursors to edit.
**
******************************************************************************/
STATIC FUNCTION GetDefault()
STATIC oDefaultWnd, oOK, lSelect
LOCAL GetList := {}
LOCAL aCursors := {"NONE", "ARROW", "HOUR", "VSIZE", "HSIZE", "NESWSIZE", "NWSESIZE"}
LOCAL nCursor

   IF oDefaultWnd==NIL

      CREATE WINDOW oDefaultWnd CENTERED ;
         SIZE 10, 26 ;
         STYLE PWSTYLE_DIALOG ;
         TITLE "Cursor Shapes" ;
         WINDOW HANDLER {|idMsg| IIF(idMsg==PWMSG_CLOSE, pw():endModal(), ), .T.}

      @ 0, 1 GET nCursor AS LISTBOX ;
         USING aCursors ;
         SIZE 8, 12 ;
         FILTER {|idMsg, _1, _2, _3, _4| ListFilter(idMsg, _1, _2, _3, _4, oDefaultWnd)}

      @ 1, 15 GET AS PUSHBUTTON oOK ;
         PROMPT "  OK  " ;
         ACTION {||lSelect := .T., SetDefault(aCursors[nCursor]), oDefaultWnd:close()}

      @ 4, 15 GET AS PUSHBUTTON ;
         PROMPT "Cancel" ;
         ACTION {||lSelect := .F., oDefaultWnd:close()} ;
         SHORTCUT K_ESC

      ATTACH CONTROLS TO oDefaultWnd DEFAULT ACTION oOK

   ELSE

      oDefaultWnd:select(oDefaultWnd:getGetList()[1])

   END

   lSelect := .F.

   OPEN WINDOW oDefaultWnd MODAL
   pw():beginModal()

RETURN(lSelect)

/****************************************************************************
** FUNCTION ListFilter()
** Parameters  -  standard 5 filter parameters
**                oWnd -> window containing list box
** Returns     -  NIL
**
** This function is a filter for the listbox control in the default cursor
** window.  It allows you to double-click on a listbox element.
**
******************************************************************************/
STATIC FUNCTION ListFilter(idMsg, _1, _2, _3, _4, oWnd)
LOCAL nCol, nRow, nWndCol, nWndRow
LOCAL oCtrlRef, oListbox

   IF idMsg==PWMSG_MOUSEDOUBLE
      oWnd:screenToWnd(_2, _3, @nWndRow, @nWndCol)
      oCtrlRef  := oWnd:ctrlAtPos(nWndRow, nWndCol)
      oListbox := oWnd:getGetList()[1]
      IF oCtrlRef == oListbox
         oListbox:wndToCtrl(nWndRow, nWndCol, @nRow, @nCol)
         IF (nCol < (oListbox:width - 3)) .AND. (nRow > 1) .AND. ;
            (nRow <= (oListbox:height - 2))
            oWnd:getGetList()[2]:event(PWMSG_KEYPRESS, K_ENTER)
         END
      END
   END

RETURN NIL

/****************************************************************************
** FUNCTION SetDefault()
** Parameters  -  cCursor -> name of cursor to load
** Returns     -  NIL
**
** This function actually loads the cursor masks into the mask arrays.
**
******************************************************************************/
STATIC FUNCTION SetDefault(cCursor)
LOCAL n
   DO CASE

      CASE cCursor == "NONE"
         FOR n := 1 to 16
            aScreenMask[n] := "11111111"
            aCursorMask[n] := "00000000"
            oXHot:setValue(0)
            oYHot:setValue(0)
         NEXT

      CASE cCursor == "ARROW"
         aScreenMask[1]  := "00111111"
         aScreenMask[2]  := "00011111"
         aScreenMask[3]  := "00001111"
         aScreenMask[4]  := "00000111"
         aScreenMask[5]  := "00000011"
         aScreenMask[6]  := "00000001"
         aScreenMask[7]  := "00000000"
         aScreenMask[8]  := "00000000"
         aScreenMask[9]  := "00000001"
         aScreenMask[10] := "00000001"
         aScreenMask[11] := "11100001"
         aScreenMask[12] := "11110000"
         aScreenMask[13] := "11110000"
         aScreenMask[14] := "11110000"
         aScreenMask[15] := "11111111"
         aScreenMask[16] := "11111111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "01000000"
         aCursorMask[3]  := "01100000"
         aCursorMask[4]  := "01110000"
         aCursorMask[5]  := "01111000"
         aCursorMask[6]  := "01111100"
         aCursorMask[7]  := "01111110"
         aCursorMask[8]  := "01111000"
         aCursorMask[9]  := "01001000"
         aCursorMask[10] := "00001100"
         aCursorMask[11] := "00001100"
         aCursorMask[12] := "00000110"
         aCursorMask[13] := "00000110"
         aCursorMask[14] := "00000000"
         aCursorMask[15] := "00000000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(1)
         oYHot:setValue(1)

      CASE cCursor == "HOUR"
         aScreenMask[1]  := "11111111"
         aScreenMask[2]  := "00000000"
         aScreenMask[3]  := "00000000"
         aScreenMask[4]  := "00000000"
         aScreenMask[5]  := "00000000"
         aScreenMask[6]  := "00000000"
         aScreenMask[7]  := "10000001"
         aScreenMask[8]  := "11000011"
         aScreenMask[9]  := "11000011"
         aScreenMask[10] := "10000001"
         aScreenMask[11] := "00000000"
         aScreenMask[12] := "00000000"
         aScreenMask[13] := "00000000"
         aScreenMask[14] := "00000000"
         aScreenMask[15] := "00000000"
         aScreenMask[16] := "11111111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "00000000"
         aCursorMask[3]  := "11111111"
         aCursorMask[4]  := "01000010"
         aCursorMask[5]  := "01000010"
         aCursorMask[6]  := "01111110"
         aCursorMask[7]  := "00111100"
         aCursorMask[8]  := "00011000"
         aCursorMask[9]  := "00011000"
         aCursorMask[10] := "00100100"
         aCursorMask[11] := "01000010"
         aCursorMask[12] := "01011010"
         aCursorMask[13] := "01111110"
         aCursorMask[14] := "11111111"
         aCursorMask[15] := "00000000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(3)
         oYHot:setValue(7)

      CASE cCursor == "VSIZE"

         aScreenMask[1]  := "11100111"
         aScreenMask[2]  := "11000011"
         aScreenMask[3]  := "10000001"
         aScreenMask[4]  := "00000000"
         aScreenMask[5]  := "00000000"
         aScreenMask[6]  := "11000011"
         aScreenMask[7]  := "11000011"
         aScreenMask[8]  := "11000011"
         aScreenMask[9]  := "11000011"
         aScreenMask[10] := "11000011"
         aScreenMask[11] := "11000011"
         aScreenMask[12] := "00000000"
         aScreenMask[13] := "00000000"
         aScreenMask[14] := "10000001"
         aScreenMask[15] := "11000011"
         aScreenMask[16] := "11100111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "00011000"
         aCursorMask[3]  := "00111100"
         aCursorMask[4]  := "01111110"
         aCursorMask[5]  := "00011000"
         aCursorMask[6]  := "00011000"
         aCursorMask[7]  := "00011000"
         aCursorMask[8]  := "00000000"
         aCursorMask[9]  := "00000000"
         aCursorMask[10] := "00011000"
         aCursorMask[11] := "00011000"
         aCursorMask[12] := "00011000"
         aCursorMask[13] := "01111110"
         aCursorMask[14] := "00111100"
         aCursorMask[15] := "00011000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(3)
         oYHot:setValue(7)

      CASE cCursor == "HSIZE"

         aScreenMask[1]  := "11100111"
         aScreenMask[2]  := "11000111"
         aScreenMask[3]  := "10000000"
         aScreenMask[4]  := "00000000"
         aScreenMask[5]  := "00000000"
         aScreenMask[6]  := "10000000"
         aScreenMask[7]  := "11000111"
         aScreenMask[8]  := "11100111"
         aScreenMask[9]  := "11100111"
         aScreenMask[10] := "11100011"
         aScreenMask[11] := "00000001"
         aScreenMask[12] := "00000000"
         aScreenMask[13] := "00000000"
         aScreenMask[14] := "00000001"
         aScreenMask[15] := "11100011"
         aScreenMask[16] := "11100111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "00010000"
         aCursorMask[3]  := "00110000"
         aCursorMask[4]  := "01111110"
         aCursorMask[5]  := "01111110"
         aCursorMask[6]  := "00110000"
         aCursorMask[7]  := "00010000"
         aCursorMask[8]  := "00000000"
         aCursorMask[9]  := "00000000"
         aCursorMask[10] := "00001000"
         aCursorMask[11] := "00001100"
         aCursorMask[12] := "01111110"
         aCursorMask[13] := "01111110"
         aCursorMask[14] := "00001100"
         aCursorMask[15] := "00001000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(3)
         oYHot:setValue(7)

      CASE cCursor == "NESWSIZE"
         aScreenMask[1]  := "11111111"
         aScreenMask[2]  := "00000000"
         aScreenMask[3]  := "10000000"
         aScreenMask[4]  := "11000000"
         aScreenMask[5]  := "10000000"
         aScreenMask[6]  := "00000000"
         aScreenMask[7]  := "00000000"
         aScreenMask[8]  := "00000000"
         aScreenMask[9]  := "00000000"
         aScreenMask[10] := "00000000"
         aScreenMask[11] := "00000000"
         aScreenMask[12] := "00000001"
         aScreenMask[13] := "00000011"
         aScreenMask[14] := "00000001"
         aScreenMask[15] := "00000000"
         aScreenMask[16] := "11111111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "00000000"
         aCursorMask[3]  := "00111110"
         aCursorMask[4]  := "00011110"
         aCursorMask[5]  := "00111110"
         aCursorMask[6]  := "01111110"
         aCursorMask[7]  := "01110010"
         aCursorMask[8]  := "00000000"
         aCursorMask[9]  := "00000000"
         aCursorMask[10] := "01001110"
         aCursorMask[11] := "01111110"
         aCursorMask[12] := "01111100"
         aCursorMask[13] := "01111000"
         aCursorMask[14] := "01111100"
         aCursorMask[15] := "00000000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(3)
         oYHot:setValue(7)

      CASE cCursor == "NWSESIZE"
         aScreenMask[1]  := "11111111"
         aScreenMask[2]  := "00000000"
         aScreenMask[3]  := "00000001"
         aScreenMask[4]  := "00000011"
         aScreenMask[5]  := "00000001"
         aScreenMask[6]  := "00000000"
         aScreenMask[7]  := "00000000"
         aScreenMask[8]  := "00000000"
         aScreenMask[9]  := "00000000"
         aScreenMask[10] := "00000000"
         aScreenMask[11] := "00000000"
         aScreenMask[12] := "10000000"
         aScreenMask[13] := "11000000"
         aScreenMask[14] := "10000000"
         aScreenMask[15] := "00000000"
         aScreenMask[16] := "11111111"
         aCursorMask[1]  := "00000000"
         aCursorMask[2]  := "00000000"
         aCursorMask[3]  := "01111100"
         aCursorMask[4]  := "01111000"
         aCursorMask[5]  := "01111100"
         aCursorMask[6]  := "01111110"
         aCursorMask[7]  := "01001110"
         aCursorMask[8]  := "00000000"
         aCursorMask[9]  := "00000000"
         aCursorMask[10] := "01110010"
         aCursorMask[11] := "01111110"
         aCursorMask[12] := "00111110"
         aCursorMask[13] := "00011110"
         aCursorMask[14] := "00111110"
         aCursorMask[15] := "00000000"
         aCursorMask[16] := "00000000"
         oXHot:setValue(3)
         oYHot:setValue(7)

   ENDCASE

   nXOldHot := HotX()
   nYOldHot := HotY()

RETURN NIL

FUNCTION OverWriteFile()
STATIC oWnd, lOverWrite
LOCAL oCancel, GetList := {}

   IF oWnd== NIL
      CREATE WINDOW oWnd CENTERED SIZE 7, 31 STYLE PWSTYLE_DIALOG ;
         TITLE "File exists" ;
         COLOR "+W/R, N/W, N/*W" ;
         WINDOW HANDLER {|idMsg| IIF(idMsg==PWMSG_CLOSE, pw():endModal(),), .T.}

      @ 1, 1 SAY PADC("Overwrite existing file?", 28)

      @ 2, 5  GET AS PUSHBUTTON PROMPT "  OK  " ACTION {||lOverWrite := .T., oWnd:close()}

      @ 2, 17 GET AS PUSHBUTTON oCancel PROMPT "Cancel" ACTION {||oWnd:close()} ;
         SHORTCUT K_ESC

      ATTACH CONTROLS TO oWnd DEFAULT ACTION oCancel INITIAL oCancel
   END

   lOverWrite  := .F.

   OPEN WINDOW oWnd MODAL
   pw():beginModal()

RETURN lOverWrite


FUNCTION AbortMessage()
LOCAL oWnd, getlist := {}

   CREATE WINDOW oWnd CENTERED SIZE 10, 50 STYLE PWSTYLE_DIALOG TITLE "Abort" ;
      COLOR "+W/R, N/W, N/*W"

   IF !pw():isTMR(PWTMR_MOUSE)
      @ 2, 1 SAY PADC("Mouse Designer requires TMR Mode", 48)
   ELSE
      @ 2, 1 SAY PADC("Mouse Designer requires a mouse", 48) //really?
   END

   @ 5, 20 GET AS PUSHBUTTON Prompt "  OK  " ;
                     ACTION {|| oWnd:close()}

   ATTACH CONTROLS TO oWnd

   OPEN WINDOW oWnd

   ACTIVATE WINDOWS
   DEINIT WINDOWS

   QUIT
RETURN NIL



