/*****************************************************************************
** FUNCTION WndMSFilter( idMsg, _1, _2, _3, _4, self ) --> NIL
**
** "Window Move-Size Filter" for ProVision:Windows v1.30 and above
**
** This neato-keeno window filter will let you (or whoever is running the
** program) move and size controls in LIVE windows using the right mouse
** button.  Any control can be moved.  Browseboxes, Checklists, Listboxes,
** Notepads, and Pushbuttons can also be sized.  (Grab the lower-right
** corner of these to begin sizing.)
**
** Here's how you use it:
**
** 	oWnd:addFilter( { | idMsg, _1, _2, _3, _4, self | ;
**			WndMSFilter( @idMsg, _1, _2, _3, _4, self ) }, ;
**			PWMSG_EVENT - PWMSG_KEYPRESS )
**
*****************************************************************************/
FUNCTION WndMSFilter( idMsg, _1, _2, _3, _4, self )

	STATIC	cScreen

	STATIC	lMove, ;
				lMoved, ;
				lTrapping	:= .F.

	STATIC	nBottom, ;
				nLastCol, ;
				nLastRow, ;
				nLeft, ;
				nRight, ;
				nTop

	STATIC	oCtrl

	LOCAL	nCol, ;
			nDeltaCol, ;
			nDeltaRow, ;
			nHeight, ;
			nRow, ;
			nSeconds, ;
			nViewHeight, ;
			nViewWidth, ;
			nWidth


	// convert screen coords to window coords
	self:screenToWnd( _2, _3, @nRow, @nCol )

	// are we interested in movement and release messages?
	IF ( lTrapping )

		// it it a movement message?
		IF ( idMsg == PWMSG_MOUSEMOVEMENT )

			lMoved		:= .T.

			// calculate delta mouse movement
			nDeltaRow	:= nRow - nLastRow
			nDeltaCol	:= nCol - nLastCol
			nLastRow		:= nRow
			nLastCol		:= nCol

			// begin display buffering
			DISPBEGIN()

			// move the control off-screen temporarily
			oCtrl:getPos( @nRow, @nCol )
			oCtrl:getSize( @nHeight, @nWidth )
			oCtrl:move( 256, nCol )

			// restore the previous screen contents
			RESTSCREEN( nTop, nLeft, nBottom, nRight, cScreen )

			// save the underlying screen contents
			nTop			:= nRow + nDeltaRow
			nLeft			:= nCol + nDeltaCol
			nBottom		:= nTop + nHeight - 1
			nRight		:= nLeft + nWidth - 1
			cScreen		:= SAVESCREEN( @nTop, @nLeft, @nBottom, @nRight )

			// do it
			IF ( lMove )
				oCtrl:move( nRow + nDeltaRow, nCol + nDeltaCol )
			ELSE
				oCtrl:move( nRow, nCol )
				oCtrl:size( nHeight + nDeltaRow, nWidth + nDeltaCol )
			END IF // ( lMove )

			// end display buffering
			DISPEND()

		// it it a right button release message?
		ELSEIF ( idMsg == PWMSG_MOUSERELEASE ) .AND. ;
				( _1 == PMBUTTON_RIGHT )

			// begin display buffering
			DISPBEGIN()

			// have all the controls redisplay themselves
			AEVAL( self:getGetList(), ;
				{ | oCtrl | oCtrl:needsRefresh := .T., oCtrl:display() } )

			// if the mouse cursor never moved, get rid of the
			// "moving" or "sizing" border
			IF ( .NOT. lMoved )

				// move the control (but not really)
				oCtrl:getPos( @nRow, @nCol )
				oCtrl:move( nRow, nCol )

			END IF // ( .NOT. lMoved )

			// end display buffering
			DISPEND()

			// unrestrict mouse cursor range
			_pmSetRange( 00, 00, ;
				( ( pwMaxCol() + 1 ) * 8 ) - 1, ;
				( ( pwMaxRow() + 1 ) * _pmFontHeight() ) - 1 )

			// we're done
			lTrapping	:= .F.
			oCtrl			:= NIL
			cScreen		:= NIL

		END IF // ( idMsg == PWMSG_MOUSEMOVEMENT )

		// no use processing this message any further
		idMsg	:= PWMSG_NONE

	ELSEIF ( idMsg == PWMSG_MOUSEPRESS ) .AND. ;
			( _1 == PMBUTTON_RIGHT )

		// get control at this pos
		oCtrl	:= self:ctrlAtPos( nRow, nCol )

		// check one row up and one row down for pushbuttons
		IF ( oCtrl == NIL )
			oCtrl	:= self:ctrlAtPos( nRow - 1, nCol )
			IF ( .NOT. ( oCtrl:className() == "pwPushbutton" ) )
				oCtrl	:= NIL
			END IF // ( .NOT. ( oCtrl:className() == "pwPushbutton" )...
		END IF // ( oCtrl == NIL )
		IF ( oCtrl == NIL )
			oCtrl	:= self:ctrlAtPos( nRow + 1, nCol )
			IF ( .NOT. ( oCtrl:className() == "pwPushbutton" ) )
				oCtrl	:= NIL
			END IF // ( .NOT. ( oCtrl:className() == "pwPushbutton" )...
		END IF // ( oCtrl == NIL )

		// is there a control there?
		IF ( oCtrl != NIL )

			// start watching for movement and release
			lTrapping	:= .T.
			lMoved		:= .F.
			nLastRow		:= nRow
			nLastCol		:= nCol

			// begin display buffering
			DISPBEGIN()

			// move the control off-screen temporarily
			oCtrl:getPos( @nRow, @nCol )
			oCtrl:getSize( @nHeight, @nWidth )
			oCtrl:move( 256, nCol )

			// save the underlying screen contents
			nTop			:= nRow
			nLeft			:= nCol
			nBottom		:= nRow + nHeight - 1
			nRight		:= nCol + nWidth - 1
			cScreen		:= SAVESCREEN( @nTop, @nLeft, @nBottom, @nRight )

			// move it back to its original position
			oCtrl:move( nRow, nCol )

			// move or size?
			IF ( ( nLastRow == ( nRow + nHeight - 1 ) ) .AND. ;
					( nLastCol == ( nCol + nWidth - 1 ) ) .AND. ;
					oCtrl:sizable )
				lMove	:= .F.
			ELSE
				lMove	:= .T.
			END IF // ( ( nLastRow == ( nRow + nHeight - 1 ) ) .AND. ...

			// display a "moving" or "sizing" border temporarily
			pwWriteBox( PWRW_TEXTATTR, ;
				nRow, nCol, ;
				nRow + nHeight - 1, nCol + nWidth - 1, ;
				"-|-|", ;
				"+W/W" )
			IF ( .NOT. lMove )
				pwWriteChar( PWRW_TEXTATTR, ;
					nRow + nHeight - 1, nCol + nWidth - 1, ;
					1, "", ;
					"+W/W" )
			END IF // ( .NOT. lMove )

			// end display buffering
			DISPEND()

			// get viewport position and size
			IF ( lMove )
				self:getVirtPos( @nViewHeight, @nViewWidth )
				self:wndToScreen( nViewHeight, nViewWidth, @nRow, @nCol )
				self:getViewSize( @nViewHeight, @nViewWidth )
			ELSE
				IF ( .NOT. ( oCtrl:className() == "pwPushbutton" ) )
					nRow	+= 3
				END IF // ( .NOT. ( oCtrl:className() == "pwPushbutton...
				nCol	+= 3
				self:getViewSize( @nViewHeight, @nViewWidth )
				nViewHeight	-= nRow
				nViewWidth	-= nCol
				self:wndToScreen( nRow, nCol, @nRow, @nCol )
			END IF // ( lMove )

			// convert screen coordinates to pixel coordinates
			nRow			*= _pmFontHeight()
			nCol			*= 8
			nViewHeight	*= _pmFontHeight()
			nViewWidth	*= 8

			// restrict mouse cursor range to window's viewport
			_pmSetRange( nCol, nRow, ;
				nCol + nViewWidth - 1, nRow + nViewHeight - 1 )

			// no use processing this message any further
			idMsg	:= PWMSG_NONE

		END IF // ( oCtrl != NIL )

	END IF // ( lTrapping )

	RETURN( NIL )

